/*
 * Copyright  2008-2009 INRIA/SensTools
 * 
 * <dev-team@sentools.info>
 * 
 * This software is a set of libraries designed to develop applications
 * for the WSN430 embedded hardware platform.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software&apos;s author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user&apos;s attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software&apos;s suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#ifndef SPI1_H
#define SPI1_H
#include "macro.h"

extern u8 spi1_tx_return_value;

enum {
    SPI1_CC1100 = 1,
    SPI1_CC2420 = 1,
    SPI1_DS1722 = 2,
    SPI1_M25P80 = 3
};

void spi1_init(void);

u8 spi1_write_single(u8 byte);
u8 spi1_read_single(void);

u8 spi1_write(u8* data, u16 len);
void spi1_read(u8* data, u16 len);

void spi1_select(u16 chip);
void spi1_deselect(u16 chip);

u16 spi1_read_somi(void);
void spi_signalpin_opendrain_active(void);
void spi_signalpin_opendrain_nonactive(void);

#endif

